@@ -30,8 +30,7 @@
 #include <polkit/polkitprivate.h>
 
 #include "polkitbackendauthority.h"
-#include "polkitbackendactionlookup.h"
-#include "polkitbackendlocalauthority.h"
+#include "polkitbackendjsauthority.h"
 
 #include "polkitbackendprivate.h"
 
@@ -40,7 +39,7 @@
  * @title: PolkitBackendAuthority
  * @short_description: Abstract base class for authority backends
  * @stability: Unstable
- * @see_also: PolkitBackendLocalAuthority
+ * @see_also: PolkitBackendJsAuthority
  *
  * To implement an authority backend, simply subclass #PolkitBackendAuthority
  * and implement the required VFuncs.
@@ -57,7 +56,7 @@ static guint signals[LAST_SIGNAL] = {0};
 G_DEFINE_ABSTRACT_TYPE (PolkitBackendAuthority, polkit_backend_authority, G_TYPE_OBJECT);
 
 static void
-polkit_backend_authority_init (PolkitBackendAuthority *local_authority)
+polkit_backend_authority_init (PolkitBackendAuthority *authority)
 {
 }
 
@@ -269,6 +268,7 @@ polkit_backend_authority_check_authorization_finish (PolkitBackendAuthority  *au
  * @subject: The subject the authentication agent wants to register for.
  * @locale: The locale of the authentication agent.
  * @object_path: The object path for the authentication agent.
+ * @options: A #GVariant with options or %NULL.
  * @error: Return location for error or %NULL.
  *
  * Registers an authentication agent.
@@ -281,6 +281,7 @@ polkit_backend_authority_register_authentication_agent (PolkitBackendAuthority
                                                         PolkitSubject             *subject,
                                                         const gchar               *locale,
                                                         const gchar               *object_path,
+                                                        GVariant                  *options,
                                                         GError                   **error)
 {
   PolkitBackendAuthorityClass *klass;
@@ -297,7 +298,7 @@ polkit_backend_authority_register_authentication_agent (PolkitBackendAuthority
     }
   else
     {
-      return klass->register_authentication_agent (authority, caller, subject, locale, object_path, error);
+      return klass->register_authentication_agent (authority, caller, subject, locale, object_path, options, error);
     }
 }
 
@@ -572,6 +573,12 @@ static const gchar *server_introspection_data =
   "      <arg type='s' name='locale' direction='in'/>"
   "      <arg type='s' name='object_path' direction='in'/>"
   "    </method>"
+  "    <method name='RegisterAuthenticationAgentWithOptions'>"
+  "      <arg type='(sa{sv})' name='subject' direction='in'/>"
+  "      <arg type='s' name='locale' direction='in'/>"
+  "      <arg type='s' name='object_path' direction='in'/>"
+  "      <arg type='a{sv}' name='options' direction='in'/>"
+  "    </method>"
   "    <method name='UnregisterAuthenticationAgent'>"
   "      <arg type='(sa{sv})' name='subject' direction='in'/>"
   "      <arg type='s' name='object_path' direction='in'/>"
@@ -874,6 +881,7 @@ server_handle_register_authentication_agent (Server                 *server,
                                                                subject,
                                                                locale,
                                                                object_path,
+                                                               NULL,
                                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -890,6 +898,63 @@ server_handle_register_authentication_agent (Server                 *server,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+server_handle_register_authentication_agent_with_options (Server                 *server,
+                                                          GVariant               *parameters,
+                                                          PolkitSubject          *caller,
+                                                          GDBusMethodInvocation  *invocation)
+{
+  GVariant *subject_gvariant;
+  GError *error;
+  PolkitSubject *subject;
+  const gchar *locale;
+  const gchar *object_path;
+  GVariant *options;
+
+  subject = NULL;
+
+  g_variant_get (parameters,
+                 "(@(sa{sv})&s&s@a{sv})",
+                 &subject_gvariant,
+                 &locale,
+                 &object_path,
+                 &options);
+
+  error = NULL;
+  subject = polkit_subject_new_for_gvariant (subject_gvariant, &error);
+  if (subject == NULL)
+    {
+      g_prefix_error (&error, "Error getting subject: ");
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
+  error = NULL;
+  if (!polkit_backend_authority_register_authentication_agent (server->authority,
+                                                               caller,
+                                                               subject,
+                                                               locale,
+                                                               object_path,
+                                                               options,
+                                                               &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
+  g_dbus_method_invocation_return_value (invocation, g_variant_new ("()"));
+
+ out:
+  if (options != NULL)
+      g_variant_unref (options);
+  if (subject != NULL)
+    g_object_unref (subject);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void
 server_handle_unregister_authentication_agent (Server                 *server,
                                                GVariant               *parameters,
@@ -1151,6 +1216,8 @@ server_handle_method_call (GDBusConnection        *connection,
     server_handle_cancel_check_authorization (server, parameters, caller, invocation);
   else if (g_strcmp0 (method_name, "RegisterAuthenticationAgent") == 0)
     server_handle_register_authentication_agent (server, parameters, caller, invocation);
+  else if (g_strcmp0 (method_name, "RegisterAuthenticationAgentWithOptions") == 0)
+    server_handle_register_authentication_agent_with_options (server, parameters, caller, invocation);
   else if (g_strcmp0 (method_name, "UnregisterAuthenticationAgent") == 0)
     server_handle_unregister_authentication_agent (server, parameters, caller, invocation);
   else if (g_strcmp0 (method_name, "AuthenticationAgentResponse") == 0)
@@ -1281,91 +1348,142 @@ polkit_backend_authority_register (PolkitBackendAuthority   *authority,
 /**
  * polkit_backend_authority_get:
  *
- * Loads all #GIOModule<!-- -->s from <filename>$(libdir)/polkit-1/extensions</filename> to determine
- * what implementation of #PolkitBackendAuthority to use. Then instantiates an object of the
- * implementation with the highest priority and unloads all other modules.
+ * Gets the #PolkitBackendAuthority to use.
  *
  * Returns: A #PolkitBackendAuthority. Free with g_object_unref().
- **/
+ */
 PolkitBackendAuthority *
 polkit_backend_authority_get (void)
 {
-  static GIOExtensionPoint *ep = NULL;
-  static GIOExtensionPoint *ep_action_lookup = NULL;
-  static volatile GType local_authority_type = G_TYPE_INVALID;
-  GList *modules;
-  GList *authority_implementations;
-  GType authority_type;
   PolkitBackendAuthority *authority;
-  gchar *s;
 
-  /* define extension points */
-  if (ep == NULL)
-    {
-      ep = g_io_extension_point_register (POLKIT_BACKEND_AUTHORITY_EXTENSION_POINT_NAME);
-      g_io_extension_point_set_required_type (ep, POLKIT_BACKEND_TYPE_AUTHORITY);
-    }
-  if (ep_action_lookup == NULL)
-    {
-      ep_action_lookup = g_io_extension_point_register (POLKIT_BACKEND_ACTION_LOOKUP_EXTENSION_POINT_NAME);
-      g_io_extension_point_set_required_type (ep_action_lookup, POLKIT_BACKEND_TYPE_ACTION_LOOKUP);
-    }
+  /* TODO: move to polkitd/main.c */
 
-  /* make sure local types are registered */
-  if (local_authority_type == G_TYPE_INVALID)
-    {
-      local_authority_type = POLKIT_BACKEND_TYPE_LOCAL_AUTHORITY;
-    }
-
-  /* load all modules */
-  modules = g_io_modules_load_all_in_directory (PACKAGE_LIB_DIR "/polkit-1/extensions");
-
-  /* find all extensions; we have at least one here since we've registered the local backend */
-  authority_implementations = g_io_extension_point_get_extensions (ep);
-
-  /* the returned list is sorted according to priority so just take the highest one */
-  authority_type = g_io_extension_get_type ((GIOExtension*) authority_implementations->data);
-  authority = POLKIT_BACKEND_AUTHORITY (g_object_new (authority_type, NULL));
-
-  /* unload all modules; the module our instantiated authority is in won't be unloaded because
-   * we've instantiated a reference to a type in this module
-   */
-  g_list_foreach (modules, (GFunc) g_type_module_unuse, NULL);
-  g_list_free (modules);
-
-  /* First announce that we've started in the generic log */
+  /* Announce that we've started in the generic log */
   openlog ("polkitd",
            LOG_PID,
            LOG_DAEMON);  /* system daemons without separate facility value */
-  syslog (LOG_INFO,
-          "started daemon version %s using authority implementation `%s' version `%s'",
-          VERSION,
-          polkit_backend_authority_get_name (authority),
-          polkit_backend_authority_get_version (authority));
+  syslog (LOG_INFO, "Started polkitd version %s", VERSION);
   closelog ();
 
-  /* and then log to the secure log */
-  s = g_strdup_printf ("polkitd(authority=%s)", polkit_backend_authority_get_name (authority));
-  openlog (s,
-           0,
+  /* then start logging to the secure log */
+  openlog ("polkitd",
+           LOG_PID,
            LOG_AUTHPRIV); /* security/authorization messages (private) */
-  /* Ugh, can't free the string - gah, thanks openlog(3) */
-  /*g_free (s);*/
+
+  authority = POLKIT_BACKEND_AUTHORITY (g_object_new (POLKIT_BACKEND_TYPE_JS_AUTHORITY, NULL));
 
   return authority;
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+typedef enum
+{
+  _COLOR_RESET,
+  _COLOR_BOLD_ON,
+  _COLOR_INVERSE_ON,
+  _COLOR_BOLD_OFF,
+  _COLOR_FG_BLACK,
+  _COLOR_FG_RED,
+  _COLOR_FG_GREEN,
+  _COLOR_FG_YELLOW,
+  _COLOR_FG_BLUE,
+  _COLOR_FG_MAGENTA,
+  _COLOR_FG_CYAN,
+  _COLOR_FG_WHITE,
+  _COLOR_BG_RED,
+  _COLOR_BG_GREEN,
+  _COLOR_BG_YELLOW,
+  _COLOR_BG_BLUE,
+  _COLOR_BG_MAGENTA,
+  _COLOR_BG_CYAN,
+  _COLOR_BG_WHITE
+} _Color;
+
+static gboolean _color_stdin_is_tty = FALSE;
+static gboolean _color_initialized = FALSE;
+
+static void
+_color_init (void)
+{
+  if (_color_initialized)
+    return;
+  _color_initialized = TRUE;
+  _color_stdin_is_tty = (isatty (STDIN_FILENO) != 0 && isatty (STDOUT_FILENO) != 0);
+}
+
+static const gchar *
+_color_get (_Color color)
+{
+  const gchar *str;
+
+  _color_init ();
+
+  if (!_color_stdin_is_tty)
+    return "";
+
+  str = NULL;
+  switch (color)
+    {
+    case _COLOR_RESET:      str="\x1b[0m"; break;
+    case _COLOR_BOLD_ON:    str="\x1b[1m"; break;
+    case _COLOR_INVERSE_ON: str="\x1b[7m"; break;
+    case _COLOR_BOLD_OFF:   str="\x1b[22m"; break;
+    case _COLOR_FG_BLACK:   str="\x1b[30m"; break;
+    case _COLOR_FG_RED:     str="\x1b[31m"; break;
+    case _COLOR_FG_GREEN:   str="\x1b[32m"; break;
+    case _COLOR_FG_YELLOW:  str="\x1b[33m"; break;
+    case _COLOR_FG_BLUE:    str="\x1b[34m"; break;
+    case _COLOR_FG_MAGENTA: str="\x1b[35m"; break;
+    case _COLOR_FG_CYAN:    str="\x1b[36m"; break;
+    case _COLOR_FG_WHITE:   str="\x1b[37m"; break;
+    case _COLOR_BG_RED:     str="\x1b[41m"; break;
+    case _COLOR_BG_GREEN:   str="\x1b[42m"; break;
+    case _COLOR_BG_YELLOW:  str="\x1b[43m"; break;
+    case _COLOR_BG_BLUE:    str="\x1b[44m"; break;
+    case _COLOR_BG_MAGENTA: str="\x1b[45m"; break;
+    case _COLOR_BG_CYAN:    str="\x1b[46m"; break;
+    case _COLOR_BG_WHITE:   str="\x1b[47m"; break;
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+  return str;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 void
 polkit_backend_authority_log (PolkitBackendAuthority *authority,
                               const gchar *format,
                               ...)
 {
+  GTimeVal now;
+  time_t now_time;
+  struct tm *now_tm;
+  gchar time_buf[128];
+  gchar *message;
   va_list var_args;
 
   g_return_if_fail (POLKIT_BACKEND_IS_AUTHORITY (authority));
 
   va_start (var_args, format);
-  vsyslog (LOG_NOTICE, format, var_args);
-
+  message = g_strdup_vprintf (format, var_args);
   va_end (var_args);
+
+  va_start (var_args, format);
+  syslog (LOG_NOTICE, "%s", message);
+
+  g_get_current_time (&now);
+  now_time = (time_t) now.tv_sec;
+  now_tm = localtime (&now_time);
+  strftime (time_buf, sizeof time_buf, "%H:%M:%S", now_tm);
+  g_print ("%s%s%s.%03d%s: %s\n",
+           _color_get (_COLOR_BOLD_ON), _color_get (_COLOR_FG_YELLOW),
+           time_buf, (gint) now.tv_usec / 1000,
+           _color_get (_COLOR_RESET),
+           message);
+
+  g_free (message);
 }
